/** @file aes.h
 * AES support routines. This file contains the AES support routine prototypes.
 *
 * @version 1.00
 *
 * Copyright (C) 2004-2010 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *  - Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AES_H
#define __AES_H

/* ==================================================================== */
/** AES data block size. This value is the number of bytes in an AES
 * data block
 */
/* ==================================================================== */
#define	AES_DATASIZE	16

/* ==================================================================== */
/** AES key size for 128 bit key. This value is the number of bytes in
 * a 128 bit AES key
 */
/* ==================================================================== */
#define	AES128_KEYSIZE		16
/* ==================================================================== */
/** AES key size for 192 bit key. This value is the number of bytes in
 * a 192 bit AES key
 */
/* ==================================================================== */
#define	AES192_KEYSIZE		24
/* ==================================================================== */
/** AES key size for 256 bit key. This value is the number of bytes in
 * a 256 bit AES key
 */
/* ==================================================================== */
#define	AES256_KEYSIZE		32

/* ==================================================================== */
/** AES number of rounds. This value is the number of rounds (-3) for
 * the corresponding encrypt / decrypt operation.  The correct value
 * for the number of rounds depends on the key size used.
 *
 * @see AES_encrypt, AES_decrypt
 */
/* ==================================================================== */
typedef enum {
    AES128_ROUNDS = 7, AES192_ROUNDS = 9, AES256_ROUNDS = 11
} AES_rounds;

#ifdef __cplusplus
extern "C" {
#endif

/* ==================================================================== */
/** 128 bit key schedule table. This type defines a key schedule
 * (either decrypt or encrypt) for the AES encryption or decryption
 * routines.  Note that both encryption and decryption schedules are
 * generated by the same routine.
 *
 * @see AES128_keySchedule()
 */
/* ==================================================================== */
typedef long long AES128_KeySchedule[22];
/** Pointer to 128 bit key schedule (either encode or decode). */
typedef AES128_KeySchedule *pAES128_KeySchedule;

/* ==================================================================== */
/** 192 bit key schedule table. This type defines a key schedule
 * (either decrypt or encrypt) for the AES encryption or decryption
 * routines.  Note that both encryption and decryption schedules are
 * generated by the same routine.
 *
 * @see AES192_keySchedule()
 */
/* ==================================================================== */
typedef long long AES192_KeySchedule[26];
/** Pointer to 192 bit key schedule (either encode or decode). */
typedef AES192_KeySchedule *pAES192_KeySchedule;

/* ==================================================================== */
/** 256 bit key schedule table. This type defines a key schedule
 * (either decrypt or encrypt) for the AES encryption or decryption
 * routines.  Note that both encryption and decryption schedules are
 * generated by the same routine.
 *
 * @see AES256_keySchedule()
 */
/* ==================================================================== */
typedef long long AES256_KeySchedule[30];
/** Pointer to 256 bit key schedule (either encode or decode). */
typedef AES256_KeySchedule *pAES256_KeySchedule;

/* ==================================================================== */
/** AES 128 bit key. This type defines a key schedule (either decrypt
 * or encrypt) for the AES encryption or decryption routines.  Note
 * that both encryption and decryption schedules are generated by the
 * same routine.
 *
 * @see AES128_keySchedule
 */
/* ==================================================================== */
typedef uint8_t AES128_Key[AES128_KEYSIZE];
/** Pointer to 128 bit key structure. */
typedef AES128_Key *pAES128_Key;

/* ==================================================================== */
/** AES 192 bit key. This type defines a key schedule (either decrypt
 * or encrypt) for the AES encryption or decryption routines.  Note
 * that both encryption and decryption schedules are generated by the
 * same routine.
 *
 * @see AES192_keySchedule
 */
/* ==================================================================== */
typedef uint8_t AES192_Key[AES192_KEYSIZE];
/** Pointer to 192 bit key structure */
typedef AES192_Key *pAES192_Key;

/* ==================================================================== */
/** AES 256 bit key. This type defines a key schedule (either decrypt
 * or encrypt) for the AES encryption or decryption routines.  Note
 * that both encryption and decryption schedules are generated by the
 * same routine.
 *
 * @see AES256_keySchedule
 */
/* ==================================================================== */
typedef uint8_t AES256_Key[AES256_KEYSIZE];
/** Pointer to 256 bit key structure */
typedef AES256_Key *pAES256_Key;

/* ==================================================================== */
/** Generate key schedules for a 128 bit key. This routine generates
 * both the encryption and decryption key schedules for a given
 * 128-bit AES key.  Each key schedule is 11 128-bit words in size, so
 * the keySchedule buffers must be able to hold 176 bytes each.  The
 * keySchedule buffers must be 64-bit aligned.
 *
 * @param[in]  key		pointer to AES key
 * @param[out] encode	pointer to generated encryption key schedule
 * @param[out] decode	pointer to generated decryption key schedule
 * 
 * @return none
 *
 * @see AES_encrypt, AES_decrypt
 */
/* ==================================================================== */

void AES128_keySchedule( pAES128_Key key, pAES128_KeySchedule encode,
			 pAES128_KeySchedule decode );

/* ==================================================================== */
/** Generate key schedules for a 192 bit key. This routine generates
 * both the encryption and decryption key schedules for a given
 * 192-bit AES key.  Each key schedule is 13 128-bit words in size, so
 * the keySchedule buffers must be able to hold 208 bytes each.  The
 * keySchedule buffers must be 64-bit aligned.
 *
 * @param[in]  key		pointer to AES key
 * @param[out] encode	pointer to generated encryption key schedule
 * @param[out] decode	pointer to generated decryption key schedule
 * 
 * @return none
 *
 * @see AES_encrypt, AES_decrypt
 */
/* ==================================================================== */
void AES192_keySchedule( pAES192_Key key, pAES192_KeySchedule encode,
			 pAES192_KeySchedule decode );

/* ==================================================================== */
/** Generate key schedules for a 256 bit key. This routine generates
 * both the encryption and decryption key schedules for a given
 * 256-bit AES key.  Each key schedule is 15 128-bit words in size, so
 * the keySchedule buffers must be able to hold 240 bytes each.  The
 * keySchedule buffers must be 64-bit aligned.
 *
 * @param[in]  key		pointer to AES key
 * @param[out] encode	pointer to generated encryption key schedule
 * @param[out] decode	pointer to generated decryption key schedule
 * 
 * @return none
 *
 * @see AES_encrypt, AES_decrypt
 */
/* ==================================================================== */
void AES256_keySchedule( pAES256_Key key, pAES256_KeySchedule encode,
			 pAES256_KeySchedule decode );

/* ==================================================================== */
/** Encrypt a 128-bit data block. This routine uses the given
 * encryption key schedule to encrypt a 128-bit block of data.  The
 * value passed for the nRounds parameter must match the decryption
 * schedule size, i.e. a schedule generated for a 128 bit key (using
 * AES128_keySchedule()) should be matched with AES128_ROUNDS for the
 * nRounds parameter, etc..
 *
 * @param[in]  plainText pointer to 128-bit plain text data input
 * @param[out] codeText	pointer to 128-bit code text data output
 * @param[in]  schedule	pointer to encryption schedule
 * @param[in]  nRounds	number of rounds - 3 (7, 9, or 11)
 * 
 * @return none
 *
 * @see AES128_keySchedule, AES192_keySchedule, AES256_keySchedule, AES_decrypt
 */
/* ==================================================================== */
void AES_encrypt( void *plainText, void *codeText, void *schedule,
		  AES_rounds nRounds );

/* ==================================================================== */
/** Decrypt a 128-bit data block. This routine uses the given
 * decryption key schedule to decrypt a 128-bit block of data.  The
 * value passed for the nRounds parameter must match the decryption
 * schedule size, i.e. a schedule generated for a 128 bit key (using
 * AES128_keySchedule()) should be matched with AES128_ROUNDS for the
 * nRounds parameter, etc..
 *
 * @param[in]  codeText	pointer to 128-bit code text data input
 * @param[out] plainText pointer to 128-bit plain text data output
 * @param[in]  schedule	pointer to decryption schedule
 * @param[in]  nRounds	number of rounds - 3 (7, 9, or 11)
 * 
 * @return none
 *
 * @see AES128_keySchedule, AES192_keySchedule, AES256_keySchedule, AES_encrypt
 */
/* ==================================================================== */
void AES_decrypt( void *codeText, void *plainText, void *schedule,
		  AES_rounds nRounds );

#ifdef __cplusplus
}
#endif

#endif /* __AES_H */
