/** @file test.c
 * AESAVS test harness. This file implements the AES ECB mode KAT and MCT tests.
 *
 * For more detailed information on the Advanced Encryption Standard
 * Algorithm Validation Suite (AESAVS), refer to the NIST website at
 *   http://csrc.nist.gov/groups/STM/cavp/documents/aes/AESAVS.pdf
 *
 * This test program only implements the Known Answer Test (KAT) and
 * Monte Carlo Test (MCT) portions of the AESAVS for ECB mode.  The AES
 * reference test vectors used can be obtained from the NIST website at
 * http://csrc.nist.gov/groups/STM/cavp/documents/aes/KAT_AES.zip (KAT data)
 * and http://csrc.nist.gov/archive/aes/rijndael/rijndael-vals.zip
 * (Monte Carlo data)
 *
 * @version 1.00
 *
 * Copyright (C) 2004-2010 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *  - Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ====================================================================
 * INCLUDE FILES
 * ==================================================================== */
/* ----------------------- documentation files ------------------------ */
#include "doxyref.h"
/* --------------------- system and platform files -------------------- */
#include <stdio.h>
#include <stdint.h>
#include <string.h>
/* -------------------------- program files --------------------------- */
#include "aes.h"
#include "testVectors.h"

/* ====================================================================
 * EXTERNAL REFERENCES
 * ==================================================================== */
/* ------------------------ data declarations ------------------------- */
extern AES_VT_VECTOR     GFSboxTests128[];
extern AES128_VK_VECTOR  keySboxTests128[];
extern AES_VT_VECTOR     varTextTests128[];
extern AES128_VK_VECTOR  varKeyTests128[];
extern AES128_ENC_VECTOR encodeTests128[];
extern AES128_DEC_VECTOR decodeTests128[];
extern AES_VT_VECTOR     GFSboxTests192[];
extern AES192_VK_VECTOR  keySboxTests192[];
extern AES_VT_VECTOR     varTextTests192[];
extern AES192_VK_VECTOR  varKeyTests192[];
extern AES192_ENC_VECTOR encodeTests192[];
extern AES192_DEC_VECTOR decodeTests192[];
extern AES_VT_VECTOR     GFSboxTests256[];
extern AES256_VK_VECTOR  keySboxTests256[];
extern AES_VT_VECTOR     varTextTests256[];
extern AES256_VK_VECTOR  varKeyTests256[];
extern AES256_ENC_VECTOR encodeTests256[];
extern AES256_DEC_VECTOR decodeTests256[];

/* ====================================================================
 * PUBLIC DECLARATIONS
 * ==================================================================== */
/* ----------------------- function prototypes ------------------------ */
void AES128_tests(void);
void AES192_tests(void);
void AES256_tests(void);
void main(void);

/* ====================================================================
 * PRIVATE DECLARATIONS
 * ==================================================================== */
/* ----------------------- function prototypes ------------------------ */
static inline void copyVector(void *, const void *);
static inline int checkVector(const void *, const void *);
static void MonteCarloEncrypt(uint8_t *, void *, AES_rounds);
static void MonteCarloDecrypt(uint8_t *, void *, AES_rounds);
static void AES128_VarTextTest(AES_VT_VECTOR *, int, char *);
static void AES128_VarKeyTest(AES128_VK_VECTOR *, int, char *);
static void AES192_VarTextTest(AES_VT_VECTOR *, int, char *);
static void AES192_VarKeyTest(AES192_VK_VECTOR *, int, char *);
static void AES256_VarTextTest(AES_VT_VECTOR *, int, char *);
static void AES256_VarKeyTest(AES256_VK_VECTOR *, int, char *);

/* ==================================================================== */
/** AES data vector zero.  This routine zeroes a single AES data block
 * (128 bits).
 * 
 * @param dst pointer to data block
 */
/* ==================================================================== */
static inline void zeroVector(void *dst)
{
    memset(dst, 0, AES_DATASIZE);
}

/* ==================================================================== */
/** AES key zero.  This routine zeroes an AES key vector.  The size of
 * the key vector can be AES128_KEYSIZE, AES192_KEYSIZE, or AES256_KEYSIZE.
 * 
 * @param dst pointer to key data block
 * @param keySize key size in bytes
 */
/* ==================================================================== */
static inline void zeroKey(void *dst, uint32_t keySize)
{
    memset(dst, 0, keySize);
}

/* ==================================================================== */
/** AES data vector copy.  This routine copies a single AES data block
 * (128 bits).
 * 
 * @param dst pointer to destination buffer
 * @param src pointer to source buffer
 */
/* ==================================================================== */
static inline void copyVector(void *dst, const void *src)
{
    memcpy(dst, src, AES_DATASIZE);
}

/* ==================================================================== */
/** AES data vector compare.  This routine compares two AES data blocks
 * (128 bits each), and returns the result of the comparison
 * 
 * @param s1 pointer to first AES data block
 * @param s2 pointer to second AES data block
 * @returns TRUE if vectors are the same
 */
/* ==================================================================== */
static inline int checkVector(const void *s1, const void *s2)
{
    return (strncmp(s1, s2, AES_DATASIZE) == 0);
}

/* ==================================================================== */
/** Monte Carlo Encryption.  This routine performs the inner set of
 * iterations for the Monte Carlo encryption test.  Basically it chains
 * the encryption of the original plaintext 10,000 times, taking the
 * output of the previous stage as the input to the next.  The final
 * result will be returned in the plaintext buffer.
 * 
 * @param plainText pointer to working buffer
 * @param schedule pointer to key schedule
 * @param nRounds number of rounds (determines key size used)
 */
/* ==================================================================== */
static void MonteCarloEncrypt(
    uint8_t *plainText, void *schedule, AES_rounds nRounds )
{
    int i;

    for (i = 0; i < 10000; i++) {
        AES_encrypt(plainText, plainText, schedule, nRounds);
    }
}

/* ==================================================================== */
/** Monte Carlo Decryption.  This routine performs the inner set of
 * iterations for the Monte Carlo decryption test.  Basically it chains
 * the decryption of the original plaintext 10,000 times, taking the
 * output of the previous stage as the input to the next.  The final
 * result will be returned in the plaintext buffer.
 * 
 * @param plainText pointer to working buffer
 * @param schedule pointer to key schedule
 * @param nRounds number of rounds (determines key size used)
 */
/* ==================================================================== */
static void MonteCarloDecrypt(
    uint8_t *plainText, void *schedule, AES_rounds nRounds )
{
    int i;

    for (i = 0; i < 10000; i++) {
        AES_decrypt(plainText, plainText, schedule, nRounds);
    }
}

/* ==================================================================== */
/** Variable Text Tests - 128-bit key.  This routine handles the variable
 * text section of the KAT tests for 128-bit keys (GFSbox and Variable
 * Text).  The key value is kept at zero, and the input data value for
 * the encryption and decryption algorithms is varied.  The result is
 * compared against the precomputed result.
 * 
 * @param pTestVec pointer to array of test vectors
 * @param numTests number of test vectors in array
 * @param testType string indicating test type ("GFSbox" or "Variable Text")
 */
/* ==================================================================== */
static void AES128_VarTextTest(
    AES_VT_VECTOR *pTestVec, int numTests, char *testType )
{
    int i;
    AES128_KeySchedule encSchedule, decSchedule;
    uint8_t resultText[AES_DATASIZE], key[AES128_KEYSIZE];
    volatile int fail = 0;
    AES_VT_VECTOR *pCurVec;

    zeroKey(key, AES128_KEYSIZE);
    AES128_keySchedule((pAES128_Key)key, &encSchedule, &decSchedule);
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES_encrypt(pCurVec->plainText, resultText, &encSchedule, AES128_ROUNDS);
        if (!checkVector(pCurVec->codeText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s encrypt: %s\n", testType, fail ? "Failed" : "Passed");

    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES_decrypt(pCurVec->codeText, resultText, &decSchedule, AES128_ROUNDS);
        if (!checkVector(pCurVec->plainText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s decrypt: %s\n", testType, fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** Variable Key Tests - 128-bit key.  This routine handles the variable
 * key section of the KAT tests for 128-bit keys (KeySbox and Variable
 * Key).  The input data value is kept constant at zero, and the key
 * value is varied.  The result of the encryption operation is compared
 * against the precomputed value.  The decryption operation starts with the
 * precomputed value and is compared against zero.
 * 
 * @param pTestVec pointer to array of test vectors
 * @param numTests number of test vectors in array
 * @param testType string indicating test type ("KeySbox" or "Variable Key")
 */
/* ==================================================================== */
static void AES128_VarKeyTest(
    AES128_VK_VECTOR *pTestVec, int numTests, char *testType )
{
    int i;
    AES128_KeySchedule encSchedule, decSchedule;
    uint8_t resultText[AES_DATASIZE], plainText[AES_DATASIZE];
    volatile int fail = 0;
    AES128_VK_VECTOR *pCurVec;

    zeroVector(plainText);      /* use zero plaintext */
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES128_keySchedule((pAES128_Key)pCurVec->key, &encSchedule, &decSchedule);
        AES_encrypt(plainText, resultText, &encSchedule, AES128_ROUNDS);
        if (!checkVector(pCurVec->codeText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s encrypt: %s\n", testType, fail ? "Failed" : "Passed");

    zeroVector(plainText);      /* use zero plaintext */
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES128_keySchedule((pAES128_Key)pCurVec->key, &encSchedule, &decSchedule);
        AES_decrypt(pCurVec->codeText, resultText, &decSchedule, AES128_ROUNDS);
        if (!checkVector(plainText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s decrypt: %s\n", testType, fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** Variable Text Tests - 192-bit key.  This routine handles the variable
 * text section of the KAT tests for 192-bit keys (GFSbox and Variable
 * Text).
 * 
 * @param pTestVec pointer to array of test vectors
 * @param numTests number of test vectors in array
 * @param testType string indicating test type ("GFSbox" or "Variable Text")
 */
/* ==================================================================== */
static void AES192_VarTextTest(
    AES_VT_VECTOR *pTestVec, int numTests, char *testType )
{
    int i;
    AES192_KeySchedule encSchedule, decSchedule;
    uint8_t resultText[AES_DATASIZE], key[AES192_KEYSIZE];
    volatile int fail = 0;
    AES_VT_VECTOR *pCurVec;

    zeroKey(key, AES192_KEYSIZE);
    AES192_keySchedule((pAES192_Key)key, &encSchedule, &decSchedule);
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES_encrypt(pCurVec->plainText, resultText, &encSchedule, AES192_ROUNDS);
        if (!checkVector(pCurVec->codeText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s encrypt: %s\n", testType, fail ? "Failed" : "Passed");

    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES_decrypt(pCurVec->codeText, resultText, &decSchedule, AES192_ROUNDS);
        if (!checkVector(pCurVec->plainText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s decrypt: %s\n", testType, fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** Variable Key Tests - 192-bit key.  This routine handles the variable
 * key section of the KAT tests for 192-bit keys (KeySbox and Variable
 * Key).
 * 
 * @param pTestVec pointer to array of test vectors
 * @param numTests number of test vectors in array
 * @param testType string indicating test type ("KeySbox" or "Variable Key")
 */
/* ==================================================================== */
static void AES192_VarKeyTest(
    AES192_VK_VECTOR *pTestVec, int numTests, char *testType )
{
    int i;
    AES192_KeySchedule encSchedule, decSchedule;
    uint8_t resultText[AES_DATASIZE], plainText[AES_DATASIZE];
    volatile int fail = 0;
    AES192_VK_VECTOR *pCurVec;

    zeroVector(plainText);      /* use zero plaintext */
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES192_keySchedule((pAES192_Key)pCurVec->key, &encSchedule, &decSchedule);
        AES_encrypt(plainText, resultText, &encSchedule, AES192_ROUNDS);
        if (!checkVector(pCurVec->codeText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s encrypt: %s\n", testType, fail ? "Failed" : "Passed");

    zeroVector(plainText);      /* use zero plaintext */
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES192_keySchedule((pAES192_Key)pCurVec->key, &encSchedule, &decSchedule);
        AES_decrypt(pCurVec->codeText, resultText, &decSchedule, AES192_ROUNDS);
        if (!checkVector(plainText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s decrypt: %s\n", testType, fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** Variable Text Tests - 256-bit key.  This routine handles the variable
 * text section of the KAT tests for 256-bit keys (GFSbox and Variable
 * Text).
 * 
 * @param pTestVec pointer to array of test vectors
 * @param numTests number of test vectors in array
 * @param testType string indicating test type ("GFSbox" or "Variable Text")
 */
/* ==================================================================== */
static void AES256_VarTextTest(
    AES_VT_VECTOR *pTestVec, int numTests, char *testType )
{
    int i;
    AES256_KeySchedule encSchedule, decSchedule;
    uint8_t resultText[AES_DATASIZE], key[AES256_KEYSIZE];
    volatile int fail = 0;
    AES_VT_VECTOR *pCurVec;

    zeroKey(key, AES256_KEYSIZE);
    AES256_keySchedule((pAES256_Key)key, &encSchedule, &decSchedule);
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES_encrypt(pCurVec->plainText, resultText, &encSchedule, AES256_ROUNDS);
        if (!checkVector(pCurVec->codeText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s encrypt: %s\n", testType, fail ? "Failed" : "Passed");

    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES_decrypt(pCurVec->codeText, resultText, &decSchedule, AES256_ROUNDS);
        if (!checkVector(pCurVec->plainText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s decrypt: %s\n", testType, fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** Variable Key Tests - 256-bit key.  This routine handles the variable
 * key section of the KAT tests for 256-bit keys (KeySbox and Variable
 * Key).
 * 
 * @param pTestVec pointer to array of test vectors
 * @param numTests number of test vectors in array
 * @param testType string indicating test type ("KeySbox" or "Variable Key")
 */
/* ==================================================================== */
static void AES256_VarKeyTest(
    AES256_VK_VECTOR *pTestVec, int numTests, char *testType )
{
    int i;
    AES256_KeySchedule encSchedule, decSchedule;
    uint8_t resultText[AES_DATASIZE], plainText[AES_DATASIZE];
    volatile int fail = 0;
    AES256_VK_VECTOR *pCurVec;

    zeroVector(plainText);      /* use zero plaintext */
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES256_keySchedule((pAES256_Key)pCurVec->key, &encSchedule, &decSchedule);
        AES_encrypt(plainText, resultText, &encSchedule, AES256_ROUNDS);
        if (!checkVector(pCurVec->codeText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s encrypt: %s\n", testType, fail ? "Failed" : "Passed");

    zeroVector(plainText);      /* use zero plaintext */
    fail = 0;
    pCurVec = pTestVec;
    for (i = 0; i < numTests; i++) {
        AES256_keySchedule((pAES256_Key)pCurVec->key, &encSchedule, &decSchedule);
        AES_decrypt(pCurVec->codeText, resultText, &decSchedule, AES256_ROUNDS);
        if (!checkVector(plainText, &resultText[0])) {
            fail++;
        }
        pCurVec++;
    }
    printf("  %s decrypt: %s\n", testType, fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** AES test harness - Key size 128 bits.  This routine handles the
 * AESAVS KAT and Monte Carlo tests for AES-128 in ECB mode.
 */
/* ==================================================================== */
void AES128_tests( void )
{
    int i;
    AES128_KeySchedule encSchedule, decSchedule;
    uint8_t plainText[AES_DATASIZE];
    volatile int fail = 0;

    printf("Key size = 128 bits\n\n");

    /* ================================================================ */
    /* ============= K n o w n   A n s w e r   t e s t s ============== */
    /* ================================================================ */
    printf("Known Answer Tests\n");
    AES128_VarTextTest(GFSboxTests128, 7, "GFSbox");
    AES128_VarKeyTest(keySboxTests128, 21, "KeySbox");
    AES128_VarTextTest(varTextTests128, 128, "Variable Text");
    AES128_VarKeyTest(varKeyTests128, 128, "Variable Key");

    /* ================================================================ */
    /* ============== M o n t e   C a r l o   t e s t s =============== */
    /* ================================================================ */
    printf("Monte Carlo Tests\n");
    fail = 0;
    for (i = 0; i < 400; i++) {
        copyVector(plainText, encodeTests128[i].plainText);
        AES128_keySchedule((pAES128_Key)encodeTests128[i].key, &encSchedule, &decSchedule);
        MonteCarloEncrypt(plainText, &encSchedule, AES128_ROUNDS);
        if (!checkVector(encodeTests128[i].codeText, &plainText[0])) {
            fail++;
        }
    }
    printf("  Encryption Tests: %s\n", fail ? "Failed" : "Passed");

    fail = 0;
    for (i = 0; i < 400; i++) {
        copyVector(plainText, decodeTests128[i].codeText);
        AES128_keySchedule((pAES128_Key)decodeTests128[i].key, &encSchedule, &decSchedule);
        MonteCarloDecrypt(plainText, &decSchedule, AES128_ROUNDS);
        if (!checkVector(decodeTests128[i].plainText, &plainText[0])) {
            fail++;
        }
    }
    printf("  Decryption Tests: %s\n\n", fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** AES test harness - Key size 192 bits.  This routine handles the
 * AESAVS KAT and Monte Carlo tests for AES-192 in ECB mode.
 */
/* ==================================================================== */
void AES192_tests( void )
{
    int i;
    AES192_KeySchedule encSchedule, decSchedule;
    uint8_t plainText[AES_DATASIZE];
    volatile int fail = 0;

    printf("Key size = 192 bits\n\n");

    /* ================================================================ */
    /* ============= K n o w n   A n s w e r   t e s t s ============== */
    /* ================================================================ */
    printf("Known Answer Tests\n");
    AES192_VarTextTest(GFSboxTests192, 6, "GFSbox");
    AES192_VarKeyTest(keySboxTests192, 24, "KeySbox");
    AES192_VarTextTest(varTextTests192, 128, "Variable Text");
    AES192_VarKeyTest(varKeyTests192, 192, "Variable Key");

    /* ================================================================ */
    /* ============== M o n t e   C a r l o   t e s t s =============== */
    /* ================================================================ */
    printf("Monte Carlo Tests\n");
    fail = 0;
    for (i = 0; i < 400; i++) {
        copyVector(plainText, encodeTests192[i].plainText);
        AES192_keySchedule((pAES192_Key)encodeTests192[i].key, &encSchedule, &decSchedule);
        MonteCarloEncrypt(plainText, &encSchedule, AES192_ROUNDS);
        if (!checkVector(encodeTests192[i].codeText, &plainText[0])) {
            fail++;
        }
    }
    printf("  Encryption Tests: %s\n", fail ? "Failed" : "Passed");

    fail = 0;
    for (i = 0; i < 400; i++) {
        copyVector(plainText, decodeTests192[i].codeText);
        AES192_keySchedule((pAES192_Key)decodeTests192[i].key, &encSchedule, &decSchedule);
        MonteCarloDecrypt(plainText, &decSchedule, AES192_ROUNDS);
        if (!checkVector(decodeTests192[i].plainText, &plainText[0])) {
            fail++;
        }
    }
    printf("  Decryption Tests: %s\n\n", fail ? "Failed" : "Passed");
}

/* ==================================================================== */
/** AES test harness - Key size 256 bits.  This routine handles the
 * AESAVS KAT and Monte Carlo tests for AES-256 in ECB mode.
 */
/* ==================================================================== */
void AES256_tests( void )
{
    int i;
    AES256_KeySchedule encSchedule, decSchedule;
    uint8_t plainText[AES_DATASIZE];
    volatile int fail = 0;

    printf("Key size = 256 bits\n\n");

    /* ================================================================ */
    /* ============= K n o w n   A n s w e r   t e s t s ============== */
    /* ================================================================ */
    printf("Known Answer Tests\n");
    AES256_VarTextTest(GFSboxTests256, 5, "GFSbox");
    AES256_VarKeyTest(keySboxTests256, 16, "KeySbox");
    AES256_VarTextTest(varTextTests256, 128, "Variable Text");
    AES256_VarKeyTest(varKeyTests256, 256, "Variable Key");

    /* ================================================================ */
    /* ============== M o n t e   C a r l o   t e s t s =============== */
    /* ================================================================ */
    printf("Monte Carlo Tests\n");
    fail = 0;
    for (i = 0; i < 400; i++) {
        copyVector(plainText, encodeTests256[i].plainText);
        AES256_keySchedule((pAES256_Key)encodeTests256[i].key, &encSchedule, &decSchedule);
        MonteCarloEncrypt(plainText, &encSchedule, AES256_ROUNDS);
        if (!checkVector(encodeTests256[i].codeText, &plainText[0])) {
            fail++;
        }
    }
    printf("  Encryption Tests: %s\n", fail ? "Failed" : "Passed");

    fail = 0;
    for (i = 0; i < 400; i++) {
        copyVector(plainText, decodeTests256[i].codeText);
        AES256_keySchedule((pAES256_Key)decodeTests256[i].key, &encSchedule, &decSchedule);
        MonteCarloDecrypt(plainText, &decSchedule, AES256_ROUNDS);
        if (!checkVector(decodeTests256[i].plainText, &plainText[0])) {
            fail++;
        }
    }
    printf("  Decryption Tests: %s\n", fail ? "Failed" : "Passed");
}

/** Physical memory location of the L2 cache configuration register */
#define L2CFG_REG	0x01840000
/** Physical memory location of the MAR register that controls
 * cacheability of external memory (MAR128 for the DaVinci EVM)
 */
#define MAR_EXTMEM	0x01848200
	
/* ==================================================================== */
/** AES test harness. This routine invokes the AESAVS KAT and Monte
 * Carlo tests for all three supported key sizes (128, 192, and 256 bits).
 */
/* ==================================================================== */

void main ( void )
{
    volatile uint32_t *mmrLoc;
	
    /* Set L2 cache size to 64K bytes
     */
    mmrLoc = (uint32_t *) L2CFG_REG;
    asm(" dint");
    *mmrLoc = (*mmrLoc & ~7) | 2;
    asm(" rint");

    /* Enable caching for external memory
     */
    mmrLoc = (uint32_t *) MAR_EXTMEM;
    *mmrLoc = 1;

    printf("NIST AES Algorithm Validation Suite\nElectronic Code Book (ECB) Mode\n");

    AES128_tests();
    AES192_tests();
    AES256_tests();
}
