/** @file testVectors.h
 * Test vector structure definitions. This file contains the test vector
 * structure definitions.
 *
 * @version 1.00
 *
 * Copyright (C) 2004-2010 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *  - Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __TESTVECTORS_H
#define __TESTVECTORS_H

#ifdef __cplusplus
extern "C" {
#endif

/* ==================================================================== */
/** Key independent test vector. This structure describes a key size
 * independent test vector.  It only specifies the plain text and the
 * corresponding code text. Used in the GFSbox and Variable Text
 * tests.
 */
/* ==================================================================== */
typedef struct {
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES_VT_VECTOR;

/* ==================================================================== */
/** Known Plain Text test vector for 128 bit keys. This structure
 * describes a test vector that specifies a 128-bit key and the
 * corresponding code text.  The plain text consists of a fixed zero
 * pattern.  Used in the KeySbox test.
 */
/* ==================================================================== */
typedef struct {
    /** Encryption key value */
    uint8_t key[AES128_KEYSIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES128_VK_VECTOR;

/* ==================================================================== */
/** Encryption test vector for 128 bit keys. This structure describes
 * a complete test vector that specifies a 128-bit key, a plain text
 * pattern, and the corresponding code text.  Used in the Monte Carlo
 * encode tests.
 */
/* ==================================================================== */
typedef struct {
    /** Encryption key value */
    uint8_t key[AES128_KEYSIZE];
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES128_ENC_VECTOR;

/* ==================================================================== */
/** Decryption test vector for 128 bit keys. This structure describes
 * a complete test vector that specifies a 128-bit key, a code text
 * pattern, and the corresponding plain text.  Used in the Monte Carlo
 * decode tests.
 */
/* ==================================================================== */
typedef struct {
    /** Decryption key value */
    uint8_t key[AES128_KEYSIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
} AES128_DEC_VECTOR;

/* ==================================================================== */
/** Known Plain Text test vector for 192 bit keys. This structure
 * describes a test vector that specifies a 192-bit key and the
 * corresponding code text.  The plain text consists of a fixed zero
 * pattern.  Used in the KeySbox test.
 */
/* ==================================================================== */
typedef struct {
    /** Encryption key value */
    uint8_t key[AES192_KEYSIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES192_VK_VECTOR;

/* ==================================================================== */
/** Encryption test vector for 192 bit keys. This structure describes
 * a complete test vector that specifies a 192-bit key, a plain text
 * pattern, and the corresponding code text.  Used in the Monte Carlo
 * encode tests.
 */
/* ==================================================================== */
typedef struct {
    /** Encryption key value */
    uint8_t key[AES192_KEYSIZE];
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES192_ENC_VECTOR;

/* ==================================================================== */
/** Decryption test vector for 192 bit keys. This structure describes
 * a complete test vector that specifies a 192-bit key, a code text
 * pattern, and the corresponding plain text.  Used in the Monte Carlo
 * decode tests.
 */
/* ==================================================================== */
typedef struct {
    /** Decryption key value */
    uint8_t key[AES192_KEYSIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
} AES192_DEC_VECTOR;

/* ==================================================================== */
/** Known Plain Text test vector for 256 bit keys. This structure
 * describes a test vector that specifies a 256-bit key and the
 * corresponding code text.  The plain text consists of a fixed zero
 * pattern.  Used in the KeySbox test.
 */
/* ==================================================================== */
typedef struct {
    /** Encryption key value */
    uint8_t key[AES256_KEYSIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES256_VK_VECTOR;

/* ==================================================================== */
/** Encryption test vector for 256 bit keys. This structure describes
 * a complete test vector that specifies a 256-bit key, a plain text
 * pattern, and the corresponding code text.  Used in the Monte Carlo
 * encode tests.
 */
/* ==================================================================== */
typedef struct {
    /** Encryption key value */
    uint8_t key[AES256_KEYSIZE];
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
} AES256_ENC_VECTOR;

/* ==================================================================== */
/** Decryption test vector for 256 bit keys. This structure describes
 * a complete test vector that specifies a 256-bit key, a code text
 * pattern, and the corresponding plain text.  Used in the Monte Carlo
 * decode tests.
 */
/* ==================================================================== */
typedef struct {
    /** Decryption key value */
    uint8_t key[AES256_KEYSIZE];
    /** Code (encrypted) data */
    uint8_t codeText[AES_DATASIZE];
    /** Plain (unencrypted) data */
    uint8_t plainText[AES_DATASIZE];
} AES256_DEC_VECTOR;

#ifdef __cplusplus
}
#endif

#endif /* __TESTVECTORS_H */
